@@ -33,7 +33,7 @@ class ArticlePool(Resource):
         thread_id = random.randint(0, 10000)
         exporting_threads[thread_id] = ExportingThread(url, language=language, translate=translate)
         exporting_threads[thread_id].start()
-        result = {"thread_id": thread_id}
-        return result, 200, {'Content-Type': 'application/json'}
+        article = exporting_threads[thread_id].article
+        return article.get_json(), 200, {'Content-Type': 'application/json'}
 
 
